package com.netease.nim.uikit.business.contact.core.util

import android.content.Context
import com.bullet.chat.grpc.multiSignupRequest
import com.bullet.chat.grpc.userProto
import com.netease.nim.uikit.R
import com.netease.nim.uikit.api.NimUIKit
import com.netease.nim.uikit.business.contact.core.item.ContactTypeHelper
import com.netease.nim.uikit.business.contact.pick.model.SelectContactItem
import com.netease.nim.uikit.business.reply.MsgServiceUtil
import com.netease.nim.uikit.common.ui.dialog.DialogCreator
import com.netease.nim.uikit.common.ui.dialog.DialogMaker
import smartisan.cloud.im.ProtoBuffManager
import smartisan.cloud.im.framwork.NimSingleThreadExecutor
import smartisan.cloud.im.log.LogUtil

object AccountBatchCreator {
    private val TAG = AccountBatchCreator::class.java.name

    fun start(context: Context, accounts: MutableList<SelectContactItem>, callback: (List<String>) -> Unit) {

        if (accounts.any { ContactTypeHelper.isPhoneContact(it) }) {
            DialogCreator.showAlert(context,
                    R.string.create_advanced_team,
                    R.string.send_sms_guide,
                    R.string.cancel,
                    R.string.continue_create,
                    object : DialogCreator.DialogClickEventCallback() {
                        override fun onNegative() {
                            startCreate(context, accounts, callback)
                        }

                        override fun onPositive() {
                        }
                    })
        } else {
            accounts.map { it.account }.let(callback)
        }

    }

    fun startCreate(context: Context, accounts: MutableList<SelectContactItem>, callback: (List<String>) -> Unit) {
        DialogMaker.showProgressDialog(context, context.getString(com.netease.nim.uikit.R.string.empty), false)
        NimSingleThreadExecutor.getInstance().execute(object : NimSingleThreadExecutor.NimTask<List<String>> {
            override fun runInBackground(): List<String> {

                val phoneContacts = mutableListOf<userProto>()
                val otherContacts = mutableListOf<String>()
                check(accounts, phoneContacts, otherContacts)

                return mutableListOf<String>().apply {
                    addAll(create(phoneContacts))
                    addAll(otherContacts)
                }
            }

            override fun onCompleted(result: List<String>?) {
                LogUtil.i(TAG, "check start -------")
                result?.forEach {
                    LogUtil.i(TAG, "account:$it")
                }
                LogUtil.i(TAG, "check end -------")

                result?.let(callback)
                DialogMaker.dismissProgressDialog()
            }
        })
    }

    fun check(accounts: MutableList<SelectContactItem>, phoneContacts: MutableList<userProto>, otherContacts: MutableList<String>) {
        LogUtil.i(TAG, "start -------")
        accounts.forEach {
            LogUtil.i(TAG, "account:${it.account} ${it.itemType}")
        }
        LogUtil.i(TAG, "end -------")



        accounts.forEach {
            if (ContactTypeHelper.isPhoneContact(it)) {
                val phoneContactModel = NimUIKit.getPhoneRecommendProvider().getPhoneContactById(it.account)
                if (phoneContactModel != null) {
                    val user = userProto.newBuilder()
                            .setCellphone(phoneContactModel.number)
                            .setNickname(phoneContactModel.name).build()
                    phoneContacts.add(user)
                } else {
                    LogUtil.i(TAG, "contact not in cache!!")
                }
            } else {
                otherContacts.add(it.account)
            }
        }

    }

    fun create(userProtos: MutableList<userProto>): MutableList<String> {
        val accounts = mutableListOf<String>()
        try {
            val stub = ProtoBuffManager.getInstance().doGetAccountStub()
            val response = stub.multiSignup(
                    multiSignupRequest.newBuilder()
                            .addAllUserProtos(userProtos)
                            .build())


            for (i in 0 until response.userProfilesOrBuilderList.count()) {
                val userProfile = response.userProfilesOrBuilderList[i]

                accounts.add(userProfile.accid)

                if (i < response.inviteUrlsList.size && userProfile.cellphone.isNotEmpty()) {

                    LogUtil.i(TAG, "LINK:" + response.inviteUrlsList[i] + "  n:" + userProfile.cellphone)
                    MsgServiceUtil.sendSmsMmsByInviteLink(userProfile.cellphone, response.inviteUrlsList[i])
                    NimUIKit.getPhoneRecommendProvider().setPhoneContact(userProfile.accid, userProfile.cellphone)
                }
            }


        } catch (e: Exception) {
            e.printStackTrace()
            LogUtil.e(TAG, e.message)
        }
        return accounts
    }
}