package com.netease.nim.uikit.business.contact.core.util

import android.content.Context
import android.view.View
import android.widget.ListView
import android.widget.Toast
import com.netease.nim.uikit.R
import com.netease.nim.uikit.api.NimUIKit
import com.netease.nim.uikit.api.model.session.SessionCustomization
import com.netease.nim.uikit.business.recent.RecentDraftHelper
import com.netease.nim.uikit.business.reply.MsgServiceUtil
import com.netease.nim.uikit.business.reply.PhoneContactMsgService
import com.netease.nim.uikit.business.session.actions.AlbumOrVideoAction
import com.netease.nim.uikit.business.session.actions.BaseAction
import com.netease.nim.uikit.business.session.actions.LocationAction
import com.netease.nim.uikit.business.session.actions.TakeVideoOrImageAction
import com.netease.nim.uikit.business.session.module.ModuleProxy
import com.netease.nim.uikit.business.session.module.input.InputPanel
import com.netease.nim.uikit.common.server.IMMsgHelper
import com.netease.nim.uikit.common.ui.toast.NimToast
import com.netease.nim.uikit.common.util.sys.ScreenUtil
import com.netease.nim.uikit.common.util.sys.UIHandler.getHandler
import com.netease.nim.uikit.impl.NimUIKitImpl
import com.netease.nimlib.sdk.NIMClient
import com.netease.nimlib.sdk.RequestCallback
import com.netease.nimlib.sdk.ResponseCode
import com.netease.nimlib.sdk.msg.MessageBuilder
import com.netease.nimlib.sdk.msg.MsgService
import com.netease.nimlib.sdk.msg.constant.MsgStatusEnum
import com.netease.nimlib.sdk.msg.constant.SessionTypeEnum
import com.netease.nimlib.sdk.msg.model.CustomMessageConfig
import com.netease.nimlib.sdk.msg.model.IMMessage
import java.util.*

/**
 * 文本快捷回复的配置类
 * 继承 ModuleProxy
 */
class InputModuleProxy(private val inputPanel: InputPanel,
                       val listView: ListView,
                       val context: Context) : ModuleProxy {

    val draftHelper = RecentDraftHelper()

    override fun sendMessage(msg: IMMessage?): Boolean {

        msg ?: return false

        if (NimUIKit.getPhoneRecommendProvider().isPhoneContact(msg.sessionId)) {
//            createSession(msg.sessionId, msg)
            PhoneContactMsgService(msg).sendMessage(msg.sessionId)
        } else {
            sendMsg(msg)
        }

        return true
    }

    /**
     * 通讯录好友快捷回复时，要先创建会话
     */
    private fun createSession(rawContactId: String, msg: IMMessage) {
        val number = NimUIKit.getPhoneRecommendProvider().getNumberByContactId(rawContactId)
        val name = NimUIKit.getPhoneRecommendProvider().getNameByContactId(rawContactId)
        PhoneContactSessionCreator.create(context, number, name, {
            val message = MessageBuilder.createTextMessage(it.userProfile?.accid, msg.sessionType, msg.content)
            sendMsg(message)
        })
    }

    private fun sendMsg(msg: IMMessage) {
        val result = MsgServiceUtil.sendMessage(msg, null, object : RequestCallback<Any> {
            override fun onSuccess(p0: Any?) {
                NimToast.makeToast(context, R.string.send_success, Toast.LENGTH_SHORT).show()
            }

            override fun onFailed(p0: Int) {
                sendFailWithBlackList(p0, msg)
            }

            override fun onException(p0: Throwable?) {
            }

        })

        if (result) {
            inputPanel.setVisiable(View.GONE)
            inputPanel.hideInputMethod()
        }
    }

    override fun onInputPanelExpand() {
    }

    override fun onEmoticonPanelCollapse() {
    }

    override fun collapseInputPanel(): Boolean {
        return inputPanel.collapse(false)
    }

    override fun isLongClickEnabled(): Boolean {
        return false
    }

    override fun markSingleReply(replyMsg: IMMessage?, replyName: String?) {
        inputPanel.setReplyMsg(replyMsg, replyName)
    }

    override fun onAvatarLongClicked(item: IMMessage?) {
        inputPanel.onAvatarLongClicked(item)
    }

    override fun revokeMessage(origMsg: IMMessage?) {
        inputPanel.reuseMsg(origMsg)
    }

    override fun onSwitchQuickReply(mVoiceQuickReply: Boolean) {
    }


    fun getActionList(customization: SessionCustomization?): List<BaseAction> {
        val actions = ArrayList<BaseAction>()
        actions.add(AlbumOrVideoAction())
        actions.add(TakeVideoOrImageAction())
        actions.add(LocationAction())

        if (customization?.actions != null) {
            actions.addAll(customization.actions)
        }
        return actions
    }

    fun getSessionCustomization(sessionTypeEnum: SessionTypeEnum): SessionCustomization {
        return if (sessionTypeEnum == SessionTypeEnum.Team)
            NimUIKitImpl.getCommonTeamSessionCustomization()
        else
            NimUIKitImpl.getCommonP2PSessionCustomization()
    }

    // 被对方拉入黑名单后，发消息失败的交互处理
    private fun sendFailWithBlackList(code: Int, msg: IMMessage) {
        if (code == ResponseCode.RES_IN_BLACK_LIST.toInt()) {
            // 如果被对方拉入黑名单，发送的消息前不显示重发红点
            msg.status = MsgStatusEnum.success
            IMMsgHelper.putMsgExtension(msg, IMMsgHelper.IS_SEND_FAILED_WITH_BL, "true")
            NIMClient.getService(MsgService::class.java).updateIMMessageStatus(msg)
            // 同时，本地插入被对方拒收的tip消息
            val tip = MessageBuilder.createTipMessage(msg.sessionId, msg.sessionType)
            tip.content = context.getString(R.string.black_list_send_tip)
            tip.status = MsgStatusEnum.success
            val config = CustomMessageConfig()
            config.enableUnreadCount = false
            tip.config = config
            NIMClient.getService(MsgService::class.java).saveMessageToLocal(tip, true)
        }
    }

    fun scrollRecycleView(touchView: View) {
        val touchViewY = ScreenUtil.getLocationOnScreen(touchView)[1] + touchView.measuredHeight
        getHandler().postDelayed({ scrollRecycleView(touchViewY, inputPanel.inputPanelLocation) }, 300)
    }

    private fun scrollRecycleView(touchViewY: Int, inputPanelY: Int) {
        if (inputPanelY in 1..(touchViewY - 1)) {
            listView.smoothScrollBy(touchViewY - inputPanelY, 100)
        }
    }

}

enum class InputType {
    Contact, Session
}